<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Form\Type;

use Dedi\SyliusVoucherPlugin\Checker\VoucherCheckerContextInterface;
use Dedi\SyliusVoucherPlugin\Controller\Action\AddVoucherToOrderCommand;
use Dedi\SyliusVoucherPlugin\Model\Voucher;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Sylius\Component\Core\Model\ShopUser;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\Security\Core\User\UserInterface;

final class AddVoucherToOrderType extends AbstractType
{
    private array $validationGroups;

    private TokenStorageInterface $tokenStorage;

    private VoucherCheckerContextInterface $checkerContext;

    private VoucherRepositoryInterface $voucherRepository;

    public function __construct(
        array $validationGroups,
        TokenStorageInterface $tokenStorage,
        VoucherCheckerContextInterface $checkerContext,
        VoucherRepositoryInterface $voucherRepository
    ) {
        $this->validationGroups = $validationGroups;
        $this->tokenStorage = $tokenStorage;
        $this->checkerContext = $checkerContext;
        $this->voucherRepository = $voucherRepository;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        /** @var ShopUser $user */
        $user = $this->tokenStorage->getToken()->getUser();

        if (!$user instanceof UserInterface) {
            throw new AccessDeniedException('This user does not have access to this section.');
        }

        $builder
            ->add('voucher', EntityType::class, [
                'class' => Voucher::class,
                'multiple' => false,
                'expanded' => true,
                'choices' => $this->checkerContext->getVouchersEligible(
                    $this->voucherRepository->findEnabledByUser($user->getCustomer())
                ),
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => AddVoucherToOrderCommand::class,
            'validation_groups' => $this->validationGroups,
        ]);
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_voucher_add_voucher_to_order';
    }
}
